#include "Shape.h"
#include <iostream>

#define EIGEN_DONT_ALIGN_STATICALLY
#include <Eigen/Dense>

#include "GLSL.h"
#include "Program.h"

#define TINYOBJLOADER_IMPLEMENTATION
#include "tiny_obj_loader.h"

using namespace std;

Shape::Shape() :
	eleBufID(0),
	posBufID(0),
	norBufID(0),
	texBufID(0), 
   vaoID(0)
{
}

Shape::~Shape()
{
}

void calcNormalPerFace(int vert1, int vert2, int vert3, vector<float> posBuf, vector<float>* normBuf, vector<int>* countBuf)
{
	float v1[3],v2[3], v3[3];
	 int x = 0;
	 int y = 1;
	 int z = 2;
	// Calculate two vectors from the three points
	float x1 = posBuf[vert1 * 3];
	float x2 = posBuf[vert2 * 3];
	float x3 = posBuf[vert3 * 3];

	float y1 = posBuf[vert1 * 3 + 1];
	float y2 = posBuf[vert2 * 3 + 1];
	float y3 = posBuf[vert3 * 3 + 1];
	
	float z1 = posBuf[vert1 * 3 + 2];
	float z2 = posBuf[vert2 * 3 + 2];
	float z3 = posBuf[vert3 * 3 + 2];

	// printf("%f %f %f\n", x1, y1, z1);
	// printf("%f %f %f\n", x2, y2, z2);
	// printf("%f %f %f\n\n", x3, y3, z3);

	// printf("Vertex Values: %f %f %f \n\n", posBuf[3 * eleBuf[i]], posBuf[3 * eleBuf[i] + 1], posBuf[3 * eleBuf[i]] + 2);		
	v1[x] = x1 - x2; 
	v1[y] = y1 - y2; 
	v1[z] = z1 - z2; 
	
	v2[x] = x2 - x3;
	v2[y] = y2 - y3; 
	v2[z] = z2 - z3;
	// Take the cross product of the two vectors to get
	// the normal vector which will be stored in out

 	v3[0] = v1[y]*v2[z] - v1[z]*v2[y];
	v3[1] = v1[z]*v2[x] - v1[x]*v2[z];
	v3[2] = v1[x]*v2[y] - v1[y]*v2[x];

	// Normalize the vector
	// float length = sqrt((v3[0] * v3[0]) 
	// 				  + (v3[1] * v3[1])
	// 				  + (v3[2] * v3[2]));
	// (*normBuf)[face + 0] += v3[0] / length;
	// (*normBuf)[face + 1] += v3[1] / length;
	// (*normBuf)[face + 2] += v3[2] / length;	

	
	//NON Normalized
	// printf("%f %f %f \n", v1[0], v1[1], v1[2]);
	// printf("%f %f %f \n", v2[0], v2[1], v2[2]);
	// printf("%f %f %f \n", v3[0], v3[1], v3[2]);

	(*normBuf)[vert1 * 3 + 0] += v3[x];
	(*normBuf)[vert1  * 3 + 1] += v3[y];
	(*normBuf)[vert1  * 3 + 2] += v3[z];	

	(*normBuf)[vert2  * 3 + 0] += v3[x];
	(*normBuf)[vert2 * 3 + 1] += v3[y];
	(*normBuf)[vert2 * 3 + 2] += v3[z];	

	(*normBuf)[vert3 * 3 + 0] += v3[x];
	(*normBuf)[vert3 * 3 + 1] += v3[y];
	(*normBuf)[vert3 * 3 + 2] += v3[z];	
}

void Shape::loadMesh(const string &meshName)
{
	// Load geometry
	// Some obj files contain material information.
	// We'll ignore them for this assignment.
	vector<tinyobj::shape_t> shapes;
	vector<tinyobj::material_t> objMaterials;
	vector<int> vertexCount;
	string errStr;
	bool rc = tinyobj::LoadObj(shapes, objMaterials, errStr, meshName.c_str());
	if(!rc) {
		cerr << errStr << endl;
	} else {
		posBuf = shapes[0].mesh.positions;
		norBuf = shapes[0].mesh.normals;
		texBuf = shapes[0].mesh.texcoords;
		eleBuf = shapes[0].mesh.indices;

		//Initialize normals to 0	
		// norBuf.resize(posBuf.size());

		// for(int i = 0; i < norBuf.size(); i++)
		// {
		// 	norBuf[i] = 0;
		// }

		// //Load the normals		 
		// for (size_t face = 0; face < eleBuf.size() - 2; face += 3)
		// {
		// 	// printf("%f %f %f\n", posBuf[face], posBuf[face + 1], posBuf[face + 2]);
		// 	calcNormalPerFace(eleBuf[face], eleBuf[face + 1], eleBuf[face + 2], posBuf, &norBuf, &vertexCount);
		// }

		// for(int i = 0; i < norBuf.size(); i+=3)
		// {
		// 	// printf("%f %f %f\n", norBuf[i], norBuf[i+1], norBuf[i + 2]);
		// }

		// printf("\n\n\n\n");
	}
}

void Shape::resize() {
  float minX, minY, minZ;
   float maxX, maxY, maxZ;
   float scaleX, scaleY, scaleZ;
   float shiftX, shiftY, shiftZ;
   float epsilon = 0.001;

   minX = minY = minZ = 1.1754E+38F;
   maxX = maxY = maxZ = -1.1754E+38F;

   //Go through all vertices to determine min and max of each dimension
   for (size_t v = 0; v < posBuf.size() / 3; v++) {
		if(posBuf[3*v+0] < minX) minX = posBuf[3*v+0];
		if(posBuf[3*v+0] > maxX) maxX = posBuf[3*v+0];

		if(posBuf[3*v+1] < minY) minY = posBuf[3*v+1];
		if(posBuf[3*v+1] > maxY) maxY = posBuf[3*v+1];

		if(posBuf[3*v+2] < minZ) minZ = posBuf[3*v+2];
		if(posBuf[3*v+2] > maxZ) maxZ = posBuf[3*v+2];
	}

	//From min and max compute necessary scale and shift for each dimension
   float maxExtent, xExtent, yExtent, zExtent;
   xExtent = maxX-minX;
   yExtent = maxY-minY;
   zExtent = maxZ-minZ;
   if (xExtent >= yExtent && xExtent >= zExtent) {
      maxExtent = xExtent;
   }
   if (yExtent >= xExtent && yExtent >= zExtent) {
      maxExtent = yExtent;
   }
   if (zExtent >= xExtent && zExtent >= yExtent) {
      maxExtent = zExtent;
   }
   scaleX = 2.0 /maxExtent;
   shiftX = minX + (xExtent/ 2.0);
   scaleY = 2.0 / maxExtent;
   shiftY = minY + (yExtent / 2.0);
   scaleZ = 2.0/ maxExtent;
   shiftZ = minZ + (zExtent)/2.0;

   //Go through all verticies shift and scale them
	for (size_t v = 0; v < posBuf.size() / 3; v++) {
		posBuf[3*v+0] = (posBuf[3*v+0] - shiftX) * scaleX;
		assert(posBuf[3*v+0] >= -1.0 - epsilon);
		assert(posBuf[3*v+0] <= 1.0 + epsilon);
		posBuf[3*v+1] = (posBuf[3*v+1] - shiftY) * scaleY;
		assert(posBuf[3*v+1] >= -1.0 - epsilon);
		assert(posBuf[3*v+1] <= 1.0 + epsilon);
		posBuf[3*v+2] = (posBuf[3*v+2] - shiftZ) * scaleZ;
		assert(posBuf[3*v+2] >= -1.0 - epsilon);
		assert(posBuf[3*v+2] <= 1.0 + epsilon);
	}
}

void Shape::init()
{
   // Initialize the vertex array object
   glGenVertexArrays(1, &vaoID);
   glBindVertexArray(vaoID);

	// Send the position array to the GPU
	glGenBuffers(1, &posBufID);
	glBindBuffer(GL_ARRAY_BUFFER, posBufID);
	glBufferData(GL_ARRAY_BUFFER, posBuf.size()*sizeof(float), &posBuf[0], GL_STATIC_DRAW);
	
	// Send the normal array to the GPU
	if(norBuf.empty()) {
		norBufID = 0;
	} else {
		glGenBuffers(1, &norBufID);
		glBindBuffer(GL_ARRAY_BUFFER, norBufID);
		glBufferData(GL_ARRAY_BUFFER, norBuf.size()*sizeof(float), &norBuf[0], GL_STATIC_DRAW);
	}
	
	// Send the texture array to the GPU
	if(texBuf.empty()) {
		texBufID = 0;
	} else {
		glGenBuffers(1, &texBufID);
		glBindBuffer(GL_ARRAY_BUFFER, texBufID);
		glBufferData(GL_ARRAY_BUFFER, texBuf.size()*sizeof(float), &texBuf[0], GL_STATIC_DRAW);
	}
	
	// Send the element array to the GPU
	glGenBuffers(1, &eleBufID);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, eleBufID);
	glBufferData(GL_ELEMENT_ARRAY_BUFFER, eleBuf.size()*sizeof(unsigned int), &eleBuf[0], GL_STATIC_DRAW);
	
	// Unbind the arrays
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
	
	assert(glGetError() == GL_NO_ERROR);
}

void Shape::draw(const shared_ptr<Program> prog) const
{
	int h_pos, h_nor, h_tex;
	h_pos = h_nor = h_tex = -1;

   glBindVertexArray(vaoID);
	// Bind position buffer
	h_pos = prog->getAttribute("vertPos");
	GLSL::enableVertexAttribArray(h_pos);
	glBindBuffer(GL_ARRAY_BUFFER, posBufID);
	glVertexAttribPointer(h_pos, 3, GL_FLOAT, GL_FALSE, 0, (const void *)0);
	
	// Bind normal buffer
	h_nor = prog->getAttribute("vertNor");
	if(h_nor != -1 && norBufID != 0) {
		GLSL::enableVertexAttribArray(h_nor);
		glBindBuffer(GL_ARRAY_BUFFER, norBufID);
		glVertexAttribPointer(h_nor, 3, GL_FLOAT, GL_FALSE, 0, (const void *)0);
	}

	if (texBufID != 0) {	
		// Bind texcoords buffer
		// h_tex = prog->getAttribute("vertTex");
		// if(h_tex != -1 && texBufID != 0) {
		// 	GLSL::enableVertexAttribArray(h_tex);
		// 	glBindBuffer(GL_ARRAY_BUFFER, texBufID);
		// 	glVertexAttribPointer(h_tex, 2, GL_FLOAT, GL_FALSE, 0, (const void *)0);
		// }
	}
	
	// Bind element buffer
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, eleBufID);
	
	// Draw
	glDrawElements(GL_TRIANGLES, (int)eleBuf.size(), GL_UNSIGNED_INT, (const void *)0);
	
	// Disable and unbind
	if(h_tex != -1) {
		GLSL::disableVertexAttribArray(h_tex);
	}
	if(h_nor != -1) {
		GLSL::disableVertexAttribArray(h_nor);
	}
	GLSL::disableVertexAttribArray(h_pos);
	glBindBuffer(GL_ARRAY_BUFFER, 0);
	glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
}
