/* Lab 5 base code - transforms using local matrix functions 
   to be written by students - 
	CPE 471 Cal Poly Z. Wood + S. Sueda
*/
#include <iostream>
#define GLEW_STATIC
#include <GL/glew.h>
#include <GLFW/glfw3.h>

#include "GLSL.h"
#include "Program.h"
#include "MatrixStack.h"
#include "Shape.h"

using namespace std;
using namespace Eigen;

GLFWwindow *window; // Main application window
string RESOURCE_DIR = ""; // Where the resources are loaded from
shared_ptr<Program> prog;
shared_ptr<Program> ground;

shared_ptr<Shape> groundPlane;
shared_ptr<Shape> hoop;
shared_ptr<Shape> ball;
shared_ptr<Shape> cube;
shared_ptr<Shape> sphere;
shared_ptr<Shape> dome;

int g_width, g_height;
float leftYRot = 0;
float testCase = 1;
int gMat = 0;
float viewRadius = 5;
float wScale = 0; //z
float uScale = 0; //x

float sTheta = -70;
float foreTheta = 30;
float handTheta = 5;
float translate = 0;
float shotDeltaY = 0;
float shotDeltaX = 0;
bool increment = true;
bool stop = false;

bool bounce = false;
bool upDrib = true;

bool ballFalling = true;

bool fireShot = false;

bool shotReady = false;
bool shotFalling = false;

float innerFingerRot = -15;
float outerFingerRot = -45;

Vector3f eye = Vector3f(-5, 5, 0.0);
Vector3f lookAtPt = Vector3f(0.0, 0.0, 0.0);
Vector3f upVector = Vector3f(0.0, 1.0, 0.0);

float mouseX = -1;
float mouseY = -1;

float yaw = 0;
float pitch = -3.14 / 4;

float curTime = 0;

float rightShouldForwardRot = -95;
float rightShouldPalmRot = 0;
float rightShouldDribRot = 0;

float leftShouldForwardRot = -85;
float leftShouldPalmRot = 0;
float leftShouldDribRot = 0;

float ballDribbleHeight = 0;
float shoulderHeight = .45;

float shotX = 0;
float shotY = 0;
float shotZ = 0;

bool ballRecovery = false;

void printMat(float *A, const char *name = 0)
{
   // OpenGL uses col-major ordering:
   // [ 0  4  8 12]
   // [ 1  5  9 13]
   // [ 2  6 10 14]
   // [ 3  7 11 15]
   // The (i,j)th element is A[i+4*j].
   if(name) {
      printf("%s=[\n", name);
   }
   for(int i = 0; i < 4; ++i) {
      for(int j = 0; j < 4; ++j) {
         printf("%- 5.2f ", A[i+4*j]);
      }
      printf("\n");
   }
   if(name) {
      printf("];");
   }
   printf("\n");
}

void reset()
{
	sTheta = -70;
	foreTheta = 30;
	handTheta = 5;
	translate = 0;
	increment = true;
	stop = false;

	bounce = false;
	upDrib = true;

	ballFalling = true;

	innerFingerRot = -15;
	outerFingerRot = -45;

	curTime = 0;

	rightShouldForwardRot = -95;
	rightShouldPalmRot = 0;
	rightShouldDribRot = 0;

	leftShouldForwardRot = -85;
	leftShouldPalmRot = 0;
	leftShouldDribRot = 0;

	ballDribbleHeight = 0;
	shoulderHeight = .45;

	shotX = 0;
	shotY = 0;
	shotZ = 0;
	fireShot = false;
	shotReady = false;
	shotFalling = false;
	shotDeltaY = 3.14 ;
}

void SetMaterial(int i) {

  switch (i) {
    case 0: //shiny blue plastic
 		glUniform3f(prog->getUniform("matAmb"), 0.02, 0.04, 0.2);
 		glUniform3f(prog->getUniform("matDif"), 0.0, 0.16, 0.9);
 		glUniform3f(prog->getUniform("matSpec"), 0.14, 0.2, 0.8);
 		glUniform1f(prog->getUniform("shine"), 120.0);
      //set shine to: (120.0);
        break;
    case 1: // flat grey
 		glUniform3f(prog->getUniform("matAmb"), 0.13, 0.13, 0.14);
 		glUniform3f(prog->getUniform("matDif"), 0.3, 0.3, 0.4);
 		glUniform3f(prog->getUniform("matSpec"), 0.3, 0.3, 0.4);
 		glUniform1f(prog->getUniform("shine"), 4.0);      
      break;
    case 2: //brass
 		glUniform3f(prog->getUniform("matAmb"), 0.3294, 0.2235, 0.02745);
 		glUniform3f(prog->getUniform("matDif"), 0.7804, 0.5686, 0.11373);
      	glUniform3f(prog->getUniform("matSpec"),0.9922, 0.941176, 0.80784);
 		glUniform1f(prog->getUniform("shine"), 27.9);    
      
        break;
	 case 3: //copper
 		glUniform3f(prog->getUniform("matAmb"), 0.1913, 0.0735, 0.0225);
 		glUniform3f(prog->getUniform("matDif"), 0.7038, 0.27048, 0.0828);
 		glUniform3f(prog->getUniform("matSpec"), 0.257, 0.1376, 0.08601);
 		glUniform1f(prog->getUniform("shine"), 12.8);         
        break;	
	 case 4: //emerald
 		glUniform3f(prog->getUniform("matAmb"), 0.0215f, 0.1745f, 0.0215f);
 		glUniform3f(prog->getUniform("matDif"), 0.07568f, 0.61424f, 0.07568f);
 		glUniform3f(prog->getUniform("matSpec"), 0.633f, 0.727811f, 0.633f);
 		glUniform1f(prog->getUniform("shine"), 76.8);         
        break;
 	 case 5: //White rubber
 		glUniform3f(prog->getUniform("matAmb"), 0.05f,0.05f,0.05f);
 		glUniform3f(prog->getUniform("matDif"), 0.5f,0.5f,0.5f);
 		glUniform3f(prog->getUniform("matSpec"),0.7f,0.7f,0.7f);
 		glUniform1f(prog->getUniform("shine"), 10.0);         
        break;
	
 	 case 6: //Shadow black
 		glUniform3f(prog->getUniform("matAmb"), 0.0f,0.0f,0.0f);
 		glUniform3f(prog->getUniform("matDif"), 0.0f,0.0f,0.0f);
 		glUniform3f(prog->getUniform("matSpec"),0.0f,0.0f,0.0f);
 		glUniform1f(prog->getUniform("shine"), 0.0);         
        break;
     case 7: //Yellow rubber
 		glUniform3f(prog->getUniform("matAmb"), 0.05f,0.05f,0.0f);
 		glUniform3f(prog->getUniform("matDif"), 0.5f,0.5f,0.4f);
 		glUniform3f(prog->getUniform("matSpec"),0.7f,0.7f,0.04f);
 		glUniform1f(prog->getUniform("shine"), 10.0);         
        break;
    
     case 8: //Red rubber
 		glUniform3f(prog->getUniform("matAmb"), 0.05f,0.05f,0.0f);
 		glUniform3f(prog->getUniform("matDif"), 0.8f,0.1f,0.1f);
 		glUniform3f(prog->getUniform("matSpec"),1.0f,0.1f,0.01f);
 		glUniform1f(prog->getUniform("shine"), 10.0);         
        break;
    }
}

void createPerspectiveMat(float *m, float fovy, float aspect, float zNear, float zFar)
{
   // http://www-01.ibm.com/support/knowledgecenter/ssw_aix_61/com.ibm.aix.opengl/doc/openglrf/gluPerspective.htm%23b5c8872587rree
   float f = 1.0f/tan(0.5f*fovy);
   m[ 0] = f/aspect;
   m[ 1] = 0.0f;
   m[ 2] = 0.0f;
   m[ 3] = 0.0f;
   m[ 4] = 0;
   m[ 5] = f;
   m[ 6] = 0.0f;
   m[ 7] = 0.0f;
   m[ 8] = 0.0f;
   m[ 9] = 0.0f;
   m[10] = (zFar + zNear)/(zNear - zFar);
   m[11] = -1.0f;
   m[12] = 0.0f;
   m[13] = 0.0f;
   m[14] = 2.0f*zFar*zNear/(zNear - zFar);
   m[15] = 0.0f;
}

static void error_callback(int error, const char *description)
{
	cerr << description << endl;
}

void zoomCamera()
{

   Vector3f w = -1 * (lookAtPt - eye);
   float wNormConst = sqrt((w.x() * w.x()) + (w.y() * w.y()) + (w.z() * w.z()));
   w = w / wNormConst;

  
   eye += wScale * w;
   lookAtPt += wScale * w;
}

void strafe()
{

   Vector3f w = -1 * (lookAtPt - eye);
   float wNormConst = sqrt((w.x() * w.x()) + (w.y() * w.y()) + (w.z() * w.z()));
   w = w / wNormConst;

   Vector3f u = upVector.cross(w);

   eye += uScale * u;
   lookAtPt += uScale * u;
}

static void key_callback(GLFWwindow *window, int key, int scancode, int action, int mods)
{
	if(key == GLFW_KEY_ESCAPE && action == GLFW_PRESS) {
		glfwSetWindowShouldClose(window, GL_TRUE);
	}	
	if(key == GLFW_KEY_A) {
		uScale = -.1;
		strafe();
	}
	if(key == GLFW_KEY_D) {
		uScale = .1;
		strafe();
	}
	if(key == GLFW_KEY_W) {
		wScale = -.1;
		zoomCamera();
	}
	if(key == GLFW_KEY_S) {
		wScale = .1;
		zoomCamera();
	}
	if(key == GLFW_KEY_SPACE && action == GLFW_PRESS) {
		stop = !stop;
	}
	if(key == GLFW_KEY_F && action == GLFW_PRESS) {
		reset();
		fireShot = true;
	}
	if(key == GLFW_KEY_R && action == GLFW_PRESS) {
		reset();
	}
}


static void mouse_callback(GLFWwindow *window, int button, int action, int mods)
{
   double posX, posY;
   if (action == GLFW_PRESS) {
      glfwGetCursorPos(window, &posX, &posY);
      // cout << "Pos X " << posX <<  " Pos Y " << posY << endl;
  }
}

static void cursor_position_callback(GLFWwindow* window, double posX, double posY)
{

	float deltaX = (posX - mouseX) / 100.0;
	float deltaY = -1 * (posY - mouseY) / 100.0;

	// printf("%f %f\n", deltaX, deltaY);
	yaw += (deltaX / 360.0) * 180 / 3.14;
	pitch += (deltaY / 180.0) * 180 / 3.14;

	if(pitch > 1.39626)
	{
		pitch = 1.39626;
	}
	else if(pitch < -1.39626)
	{
		pitch = -1.39626;
	}
	// printf("yaw: %f, pitch: %f\n\n", yaw, pitch);
	
	lookAtPt.x() = eye.x() + viewRadius * cos(yaw) * cos(pitch);
	lookAtPt.y() = eye.y() + viewRadius * sin(pitch);
	lookAtPt.z() = eye.z() + viewRadius * cos(pitch) * cos(3.14/2 - yaw);		

	mouseX = posX;
	mouseY = posY;
}


static void resize_callback(GLFWwindow *window, int width, int height) {
   g_width = width;
   g_height = height;
   glViewport(0, 0, width, height);
}

static void init()
{
	GLSL::checkVersion();

	// Set background color.
	glClearColor(.12f, .34f, .56f, 1.0f);
	// Enable z-buffer test.
	glEnable(GL_DEPTH_TEST);

	// Initialize mesh.
	

	groundPlane = make_shared<Shape>();
	groundPlane->loadMesh(RESOURCE_DIR + "ground.obj");
	groundPlane->resize();
	groundPlane->init();

	hoop = make_shared<Shape>();
	hoop->loadMesh(RESOURCE_DIR + "basketball_hoop.obj");
	hoop->resize();
	hoop->init();

	ball = make_shared<Shape>();
	ball->loadMesh(RESOURCE_DIR + "basketball.obj");
	ball->resize();
	ball->init();

	cube = make_shared<Shape>();
	cube->loadMesh(RESOURCE_DIR + "cube.obj");
	cube->resize();
	cube->init();

	sphere = make_shared<Shape>();
	sphere->loadMesh(RESOURCE_DIR + "sphere.obj");
	sphere->resize();
	sphere->init();

	dome = make_shared<Shape>();
	dome->loadMesh(RESOURCE_DIR + "dome.obj");
	dome->resize();
	dome->init();

	// Initialize the GLSL program.
	prog = make_shared<Program>();
	prog->setVerbose(true);
	prog->setShaderNames(RESOURCE_DIR + "phong_vert.glsl", RESOURCE_DIR + "phong_frag.glsl");
	prog->init();
	prog->addUniform("P");
	prog->addUniform("M");
	prog->addUniform("V");
	prog->addUniform("matAmb");
	prog->addUniform("matDif");
	prog->addUniform("matSpec");
	prog->addUniform("shine");
	prog->addUniform("eyePos");
	prog->addAttribute("vertPos");
	prog->addAttribute("vertNor");

	ground = make_shared<Program>();
	ground->setVerbose(true);
	ground->setShaderNames(RESOURCE_DIR + "ground_vert.glsl", RESOURCE_DIR + "ground_frag.glsl");
	ground->init();
	ground->addUniform("P");
	ground->addUniform("M");
	ground->addUniform("V");
	ground->addUniform("color");
}

static void render()
{
	//local modelview matrix use this for lab 5
	auto M = make_shared<MatrixStack>();
	auto V = make_shared<MatrixStack>();
   float P[16] = {0};

   V->loadIdentity();
   V->lookAt(eye, lookAtPt, upVector);
	// Get current frame buffer size.
	int width, height;
	glfwGetFramebufferSize(window, &width, &height);
	glViewport(0, 0, width, height);

	// Clear framebuffer.
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   float aspect = width/(float)height;
   createPerspectiveMat(P, 70.0f, aspect, 0.1, 100.0f);	

   float upperLegRotation = -10;
   float lowerLegRotation = 10;
   float footRotation = 10;



   	//Court floor
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, 0, 0));
		M->scale(Vector3f(viewRadius / 2, viewRadius, viewRadius));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
	// glUniform3f(prog->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  	
	glUniform3f(ground->getUniform("color"), 0.5f,0.5f,0.4f);
  	// SetMaterial(7);
	groundPlane->draw(prog);

	//Center line
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .05, 0));
		M->scale(Vector3f(viewRadius / 2 - .01, 1, .01));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  	
glUniform3f(ground->getUniform("color"), 1.0f, 1.0f, 1.0f);
  	SetMaterial(7);
	groundPlane->draw(prog);


	//Center Court Circle
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .001, 0));
		M->scale(Vector3f(viewRadius / 8, .001, viewRadius / 8));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z()); 
 	glUniform3f(ground->getUniform("color"), 1.0f, 1.0f, 1.0f);
	sphere->draw(prog);

	//Center inner Court Circle
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .004, 0));
		M->scale(Vector3f(viewRadius / 8 -.04, .002, viewRadius / 8 -.04));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
	// glUniform3f(prog->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  	
  	// SetMaterial(8);
  	glUniform3f(ground->getUniform("color"), 1.0f, 0.0f, 0.0f);
	sphere->draw(prog);


	//Left inner 3pt
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .005, -viewRadius / 2 - 1.2));
	 	// M->rotate(90, Vector3f(0, 0, 0));
		M->scale(Vector3f(viewRadius / 2 -.04, .001, viewRadius / 2 -.04));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  
	glUniform3f(ground->getUniform("color"), 0.5f,0.5f,0.4f);
	dome->draw(prog);

	//Left outer 3pt
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .004, -viewRadius / 2 - 1.2));
	 	// M->rotate(90, Vector3f(0, 0, 0));
		M->scale(Vector3f(viewRadius / 2, .001, viewRadius / 2));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  
	glUniform3f(ground->getUniform("color"), 1.0f, 1.0f, 1.0f);
	dome->draw(prog);


	//Right inner 3pt
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .005, viewRadius / 2 + 1.2));
	 	M->rotate(180, Vector3f(1, 0, 0));
		M->scale(Vector3f(viewRadius / 2 -.04, .001, viewRadius / 2 -.04));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  	
  	glUniform3f(ground->getUniform("color"), 0.5f,0.5f,0.4f);
	dome->draw(prog);

	//Right outer 3pt
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .004, viewRadius / 2 + 1.2));
	 	M->rotate(180, Vector3f(1, 0, 0));
		M->scale(Vector3f(viewRadius / 2, .001, viewRadius / 2));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  
	glUniform3f(ground->getUniform("color"), 1.0f, 1.0f, 1.0f);
	dome->draw(prog);








	//Right inner free square
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .007, viewRadius / 8 + 3.5));
	 	M->rotate(180, Vector3f(1, 0, 0));
		M->scale(Vector3f(viewRadius / 6 -.04, .001, viewRadius / 6 -.04));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  
	glUniform3f(ground->getUniform("color"), 0.5f,0.5f,0.4f);
	cube->draw(prog);

	//Right outer free square
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .006, viewRadius / 8 + 3.5));
	 	M->rotate(180, Vector3f(1, 0, 0));
		M->scale(Vector3f(viewRadius / 6, .001, viewRadius / 6));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  	
  	glUniform3f(ground->getUniform("color"), 1.0f, 1.0f, 1.0f);
	cube->draw(prog);





	//left inner free square
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .007, -viewRadius / 8 - 3.5));
	 	M->rotate(180, Vector3f(1, 0, 0));
		M->scale(Vector3f(viewRadius / 6 -.04, .001, viewRadius / 6 -.04));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z()); 
 	glUniform3f(ground->getUniform("color"), 0.5f,0.5f,0.4f);
	cube->draw(prog);

	//left outer free square
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .006, -viewRadius / 8 - 3.5));
	 	M->rotate(180, Vector3f(1, 0, 0));
		M->scale(Vector3f(viewRadius / 6, .001, viewRadius / 6));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  	
  	glUniform3f(ground->getUniform("color"), 1.0f, 1.0f, 1.0f);
	cube->draw(prog);






	//Left inner free semi circle
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .011, -viewRadius / 2 - .4));
	 	// M->rotate(90, Vector3f(0, 0, 0));
		M->scale(Vector3f(viewRadius / 6 -.04, .001, viewRadius / 6 -.04));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  	
  	glUniform3f(ground->getUniform("color"), 1.f,.1f,.1f);
	dome->draw(prog);


	//Left outer free semi circle
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .01, -viewRadius / 2 - .4));
	 	// M->rotate(90, Vector3f(0, 0, 0));
		M->scale(Vector3f(viewRadius / 6, .001, viewRadius / 6));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  	
  	glUniform3f(ground->getUniform("color"), 1.0f, 1.0f, 1.0f);
	dome->draw(prog);




	//Right inner free semi circle
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .011, viewRadius / 2 + .4));
	 	M->rotate(180, Vector3f(1, 0, 0));
		M->scale(Vector3f(viewRadius / 6 -.04, .001, viewRadius / 6 -.04));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z());  	
  	glUniform3f(ground->getUniform("color"), 1.f,.1f,.1f);
	dome->draw(prog);


	//Left outer free semi circle
	ground->bind();
	M->loadIdentity();
	 M->pushMatrix();	      
	 	M->translate(Vector3f(0, .01, viewRadius / 2 + .4));
	 	M->rotate(180, Vector3f(0, 0, 0));
		M->scale(Vector3f(viewRadius / 6, .001, viewRadius / 6));
	// Draw mesh using GLSL.
	glUniformMatrix4fv(ground->getUniform("P"), 1, GL_FALSE, P);
  	glUniformMatrix4fv(ground->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
  	glUniformMatrix4fv(ground->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
// glUniform3f(ground->getUniform("eyePos"), eye.x(), eye.y(), eye.z()); 
 	glUniform3f(ground->getUniform("color"), 1.0f, 1.0f, 1.0f);
	dome->draw(prog);





	//left hoop
	prog->bind();
	M->loadIdentity();
	 M->pushMatrix();	      	
		M->translate(Vector3f(0, 1, -viewRadius + 1));
		M->rotate(0, Vector3f(0, 1, 0));		
	// Draw mesh using GLSL.
		glUniformMatrix4fv(prog->getUniform("P"), 1, GL_FALSE, P);
	  	glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
	  	glUniformMatrix4fv(prog->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
		SetMaterial(5);
		hoop->draw(prog);
	  M->popMatrix();



	//right hoop
	prog->bind();
	M->loadIdentity();
	 M->pushMatrix();	      	
		M->translate(Vector3f(0, 1, viewRadius - 1));
		M->rotate(180, Vector3f(0, 1, 0));		
	// Draw mesh using GLSL.
		glUniformMatrix4fv(prog->getUniform("P"), 1, GL_FALSE, P);
	  	glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
	  	glUniformMatrix4fv(prog->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
		SetMaterial(5);
		hoop->draw(prog);
	  M->popMatrix();	  
 	
 	//WAVING GUY 
	// Draw a stack of cubes with indiviudal transforms 
	prog->bind();
	glUniformMatrix4fv(prog->getUniform("P"), 1, GL_FALSE, P);

	/* draw cubes */	
   prog->bind();
 
   SetMaterial(2);

   M->pushMatrix();
     M->loadIdentity();     
	  //Body
		M->translate(Vector3f(0, shoulderHeight, 0));	  	
	  	M->rotate(0, Vector3f(0, 1, 0));	
	  	M->scale(Vector3f(0.1, 0.1, 0.1));	
	  	M->pushMatrix();
	  	glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
	  	cube->draw(prog);

	  	//hips
	  	M->pushMatrix();
		  	M->translate(Vector3f(0, -1.3, 0));
		  	
		  	//left upper leg
		  	M->pushMatrix();
		  		M->rotate(upperLegRotation, Vector3f(1, 0, 0));	
		  		M->translate(Vector3f(-.75, -.75, 0));

				//left lower leg
		  		M->pushMatrix();
		  			M->translate(Vector3f(0, -.75, 0));
		  			M->rotate(lowerLegRotation, Vector3f(1, 0, 0));	
		  			M->translate(Vector3f(0, -.75, 0));
		  			
					//left foot
			  		M->pushMatrix();
			  			M->translate(Vector3f(0, -.35, .25));
			  			M->rotate(footRotation, Vector3f(1, 0, 0));	
			  			M->translate(Vector3f(0, -.25, 0));
			  			M->scale(Vector3f(.4, .25, .6));	
			  	
			  			glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			sphere->draw(prog);
			  		M->popMatrix();

		  			M->scale(Vector3f(.25, .75, .25));	
		  	
		  			glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
		  			cube->draw(prog);
		  		M->popMatrix();

		  		M->scale(Vector3f(.25, .75, .25));	
		  		glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
		  		cube->draw(prog);
		  	M->popMatrix();		


		  	//right upper leg
		  	M->pushMatrix();
		  		M->rotate(upperLegRotation, Vector3f(1, 0, 0));	
		  		M->translate(Vector3f(.75, -.75, 0));

				//right lower leg
		  		M->pushMatrix();
		  			M->translate(Vector3f(0, -.75, 0));
		  			M->rotate(lowerLegRotation, Vector3f(1, 0, 0));	
		  			M->translate(Vector3f(0, -.75, 0));
		  			
					//right foot
			  		M->pushMatrix();
			  			M->translate(Vector3f(0, -.35, .25));
			  			M->rotate(footRotation, Vector3f(1, 0, 0));	
			  			M->translate(Vector3f(0, -.25, 0));
			  			M->scale(Vector3f(.4, .25, .6));	
			  	
			  			glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			sphere->draw(prog);
			  		M->popMatrix();

		  			M->scale(Vector3f(.25, .75, .25));	
		  	
		  			glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
		  			cube->draw(prog);
		  		M->popMatrix();

		  		M->scale(Vector3f(.25, .75, .25));	
		  		glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
		  		cube->draw(prog);
		  	M->popMatrix();		

		  	M->scale(Vector3f(1, .25, 1));
		  	glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
		  	cube->draw(prog);
	  	M->popMatrix();

	  	//Head
	  	M->pushMatrix();
		  	M->translate(Vector3f(0, 2, 0));
		  	M->scale(Vector3f(.75, .75, .75));
		  	glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
		  	sphere->draw(prog);
	  	M->popMatrix();

	  	//Neck
	  	M->pushMatrix();
		  	M->translate(Vector3f(0, 1.15, 0));
		  	M->scale(Vector3f(0.15, 0.15, 0.15));
		  	glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
		  	cube->draw(prog);
	  	M->popMatrix();

		//RIGHT ARM
     	M->pushMatrix();
			//place at shoulder
			M->translate(Vector3f(1, .5, 0));

			M->rotate(rightShouldDribRot, Vector3f(1, 0, 0));
		    //Point shoulder forward		    
		    M->rotate(rightShouldForwardRot, Vector3f(0, 1, 0));
		    //Turn hand inwards
		    M->rotate(rightShouldPalmRot, Vector3f(1, 0, 0));
			//move to shoulder joint
			M->translate(Vector3f(.8, 0, 0));

			//Right Forearm
		  	M->pushMatrix();			
				M->translate(Vector3f(0.5, 0, 0));		
			    M->rotate(foreTheta, Vector3f(0, 0, 1));		
				M->translate(Vector3f(0.5, 0, 0));		
				
				//Right hand
				M->pushMatrix();				
					M->translate(Vector3f(0.5, 0, 0));				
			    	M->rotate(handTheta, Vector3f(0, 0, 1));		
					M->translate(Vector3f(0.3, 0, 0));
					
					//Basketball
					M->pushMatrix();			
						M->translate(Vector3f(shotX, shotY, shotZ));
					  	M->translate(Vector3f(0, 0, ballDribbleHeight + .7));
					    M->scale(Vector3f(.6, .6, .6));

						glUniformMatrix4fv(prog->getUniform("P"), 1, GL_FALSE, P);
					  	glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
					  	glUniformMatrix4fv(prog->getUniform("V"), 1, GL_FALSE, V->topMatrix().data());
						SetMaterial(3);//COPPER

						ball->draw(prog);
		  			M->popMatrix();
		  			SetMaterial(2);



					//Pinky Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.2, -.18, 0));				
			    		//Spacing between fingers
			    		M->rotate(-30, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(innerFingerRot, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.2, 0, 0));
					
						//Pinky Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.1, 0, 0));				
			    			M->rotate(outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.1, 0, 0));

							M->scale(Vector3f(0.1, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.15, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();

					//Ring Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.2, -.1, 0));				
			    		//Spacing between fingers
			    		M->rotate(-15, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(innerFingerRot, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.2, 0, 0));
					
						//Ring Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.2, 0, 0));				
			    			M->rotate(outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.1, 0, 0));

							M->scale(Vector3f(0.1, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.2, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();

					//Middle Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.2, 0, 0));				
			    		//Spacing between fingers
			    		M->rotate(0, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(innerFingerRot, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.2, 0, 0));
					
						//Middle Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.2, 0, 0));				
			    			M->rotate(outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.1, 0, 0));

							M->scale(Vector3f(0.1, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.25, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();

					//Index Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.2, .1, 0));				
			    		//Spacing between fingers
			    		M->rotate(15, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(innerFingerRot, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.2, 0, 0));
					
						//Index Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.2, 0, 0));				
			    			M->rotate(outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.1, 0, 0));

							M->scale(Vector3f(0.1, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.2, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();			


					//Thumb Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.11, .2, 0));				
			    		//Spacing between fingers
			    		M->rotate(90, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(-45, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.11, 0, 0));
					
						//Thumb Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.06, 0, 0));				
			    			M->rotate(outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.06, 0, 0));

							M->scale(Vector3f(0.06, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.11, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();


					M->scale(Vector3f(0.3, 0.25, 0.25));
					glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  		cube->draw(prog);
				M->popMatrix();
			
				M->scale(Vector3f(.6, 0.25, 0.25));
				//non-uniform scale
			  	glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  	cube->draw(prog);
			M->popMatrix();

		M->scale(Vector3f(.8, 0.25, 0.25));
		glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
		cube->draw(prog);	   		
		M->popMatrix();


		//Left ARM
     	M->pushMatrix();
			//place at shoulder
			M->translate(Vector3f(-1, .5, 0));

			M->rotate(leftShouldDribRot, Vector3f(1, 0, 0));
		    //Point shoulder forward		    
		    M->rotate(leftShouldForwardRot, Vector3f(0, 1, 0));
		    //Turn hand inwards
		    M->rotate(leftShouldPalmRot, Vector3f(1, 0, 0));
			//move to shoulder joint
			M->translate(Vector3f(.8, 0, 0));

			//Left Forearm
		  	M->pushMatrix();			
				M->translate(Vector3f(0.5, 0, 0));		
			    M->rotate(foreTheta, Vector3f(0, 0, 1));		
				M->translate(Vector3f(0.5, 0, 0));		
				
				//Left hand
				M->pushMatrix();				
					M->translate(Vector3f(0.5, 0, 0));				
			    	M->rotate(handTheta, Vector3f(0, 0, 1));		
					M->translate(Vector3f(0.3, 0, 0));

					//Pinky Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.2, -.18, 0));				
			    		//Spacing between fingers
			    		M->rotate(-30, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(-innerFingerRot, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.2, 0, 0));
					
						//Pinky Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.1, 0, 0));				
			    			M->rotate(-outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.1, 0, 0));

							M->scale(Vector3f(0.1, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.15, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();

					//Ring Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.2, -.1, 0));				
			    		//Spacing between fingers
			    		M->rotate(-15, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(-innerFingerRot, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.2, 0, 0));
					
						//Ring Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.2, 0, 0));				
			    			M->rotate(-outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.1, 0, 0));

							M->scale(Vector3f(0.1, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.2, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();

					//Middle Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.2, 0, 0));				
			    		//Spacing between fingers
			    		M->rotate(0, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(-innerFingerRot, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.2, 0, 0));
					
						//Middle Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.2, 0, 0));				
			    			M->rotate(-outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.1, 0, 0));

							M->scale(Vector3f(0.1, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.25, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();

					//Index Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.2, .1, 0));				
			    		//Spacing between fingers
			    		M->rotate(15, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(-innerFingerRot, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.2, 0, 0));
					
						//Index Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.2, 0, 0));				
			    			M->rotate(-outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.1, 0, 0));

							M->scale(Vector3f(0.1, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.2, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();			


					//Thumb Inner
					M->pushMatrix();				
						M->translate(Vector3f(0.11, .2, 0));				
			    		//Spacing between fingers
			    		M->rotate(90, Vector3f(0, 0, 1));		
			    	    //Gripping rotation
			    	    M->rotate(45, Vector3f(0, 1, 0));		
						M->translate(Vector3f(0.11, 0, 0));
					
						//Thumb Outer
						M->pushMatrix();				
							M->translate(Vector3f(0.06, 0, 0));				
			    			M->rotate(-outerFingerRot, Vector3f(0, 1, 0));		
							M->translate(Vector3f(0.06, 0, 0));

							M->scale(Vector3f(0.06, 0.05, 0.05));
							glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  				cube->draw(prog);
						M->popMatrix();

						M->scale(Vector3f(0.11, 0.05, 0.05));
						glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  			cube->draw(prog);
					M->popMatrix();


					M->scale(Vector3f(0.3, 0.25, 0.25));
					glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  		cube->draw(prog);
				M->popMatrix();
			
				M->scale(Vector3f(.6, 0.25, 0.25));
				//non-uniform scale
			  	glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
			  	cube->draw(prog);
			M->popMatrix();

		M->scale(Vector3f(.8, 0.25, 0.25));
		glUniformMatrix4fv(prog->getUniform("M"), 1, GL_FALSE, M->topMatrix().data());
		cube->draw(prog);	   		
		M->popMatrix();


   M->popMatrix();
   
	prog->unbind();
 
	if(!stop)
	{
		if(fireShot)
		{	
			if(foreTheta < 50 && !shotReady)
			{
				foreTheta += 1.5;
				leftShouldDribRot -= .5;
				rightShouldDribRot -= .5;
			}
			else if(leftShouldDribRot > -30)
			{
				shotReady = true;
				leftShouldDribRot--;
				rightShouldDribRot--;
			}
			// else if(foreTheta < 60)			
			// {
			// 	foreTheta++;
			// }
			else if(shotDeltaY > .34 && !shotFalling && !ballRecovery)			
			{
				// shotX += .5 - shotDeltaX;
				shotX = 0 + 21 * sin(shotDeltaY);// - shotDeltaY;
				shotY = 21 + 21 * cos(shotDeltaY);

				shotDeltaY -= 0.01;
			}
			else if(shotDeltaY < 3.14)
			{
				ballRecovery = true;
				shotDeltaY += 0.01;

				shotX = 0 + 21 * sin(shotDeltaY);// - shotDeltaY;
				shotY = 21 + 21 * cos(shotDeltaY);
			}
			else
			{
				ballRecovery = false;
			}
		}
	 	else if(rightShouldForwardRot < -60)
	 	{
	 		rightShouldForwardRot += .5;
	 		rightShouldPalmRot += 1.5;
	 	}
	 	else if(!bounce)
	 	{
	 		if(rightShouldDribRot < 15 && upDrib)
	 		{
	 			rightShouldDribRot += .5;
	 			if(rightShouldDribRot == 15)
	 			{
	 				bounce = true;
	 			}
	 		}
	 		else if(rightShouldDribRot > 0)
	 		{
	 			upDrib = false;
	 			rightShouldDribRot -= .5;
	 		}
	 		else 
	 		{
	 			upDrib = true;
	 		}
	 	}
	 	else 
	 	{
	 		if(ballDribbleHeight <= 3.5 && ballFalling)
	 		{
				// printf("Incrementing ball height: %f\n", ballDribbleHeight); 	
	 			ballDribbleHeight += .2;
	 		}
	 		else if(ballDribbleHeight > 0)
	 		{
	 			ballFalling = false;
	 			// printf("Decrementing ball height: %f\n", ballDribbleHeight); 	
	 			ballDribbleHeight -= .15;
	 		}
	 		else
	 		{
	 			ballFalling = true;
	 			bounce = false;
	 		}
	 	}
	}
}

int main(int argc, char **argv)
{
	if(argc < 2) {
		cout << "Please specify the resource directory." << endl;
		return 0;
	}
	RESOURCE_DIR = argv[1] + string("/");

	// Set error callback.
	glfwSetErrorCallback(error_callback);
	// Initialize the library.
	if(!glfwInit()) {
		return -1;
	}
   //request the highest possible version of OGL - important for mac
   glfwWindowHint(GLFW_OPENGL_FORWARD_COMPAT, GL_TRUE);
   glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);
   glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
   glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 2);

	lookAtPt.x() = eye.x() + viewRadius * cos(yaw) * cos(pitch);
	lookAtPt.y() = eye.y() + viewRadius * sin(pitch);
	lookAtPt.z() = eye.z() + viewRadius * cos(pitch) * cos(3.14/2 - yaw);		
	// Create a windowed mode window and its OpenGL context.
	window = glfwCreateWindow(640, 480, "Corrigan Johnson", NULL, NULL);
	if(!window) {
		glfwTerminate();
		return -1;
	}
	// Make the window's context current.
	glfwMakeContextCurrent(window);
	// Initialize GLEW.
	glewExperimental = true;
	if(glewInit() != GLEW_OK) {
		cerr << "Failed to initialize GLEW" << endl;
		return -1;
	}
	//weird bootstrap of glGetError
   glGetError();
	cout << "OpenGL version: " << glGetString(GL_VERSION) << endl;
   cout << "GLSL version: " << glGetString(GL_SHADING_LANGUAGE_VERSION) << endl;

	// Set vsync.
	glfwSwapInterval(1);
	// Set keyboard callback.
	glfwSetKeyCallback(window, key_callback);
   //set the mouse call back
   glfwSetMouseButtonCallback(window, mouse_callback);
   //set the window resize call back
   glfwSetFramebufferSizeCallback(window, resize_callback);
	glfwSetCursorPosCallback(window, cursor_position_callback);
	glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_DISABLED);
	// Initialize scene. Note geometry initialized in init now
	init();

	// Loop until the user closes the window.
	while(!glfwWindowShouldClose(window)) {
		// Render scene.
		curTime += 0.001;
		render();
		// Swap front and back buffers.
		glfwSwapBuffers(window);
		// Poll for and process events.
		glfwPollEvents();
	}
	// Quit program.
	glfwDestroyWindow(window);
	glfwTerminate();
	return 0;
}
